/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Tim Connolly
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:		Tiff.h
//
// COMMENTS:
//		 The Tiff File Reader
*/

#include <CPI/CPI.h>
#include <libtiff/tiffiop.h>

typedef struct _imageState
{
	CPI_DeviceHandle	 myDevice;
	TIFF				*tif;
	CPI_Uint16			 photoMetric;
	CPI_Uint16			 orientation;
	CPI_Int32			 rows;
	CPI_Int32			 columns;
	CPI_Uint16			 samplesperpixel;
	CPI_Uint16			 bitspersample;
	CPI_Uint16			 compression;
	CPI_Uint32			 lastY;
} imageState;

static CPI_Bool			 TiffInit( CPI_Bool initforread );
static CPI_Bool			 TiffShutDown( void );
static void				 TiffParmChanged( const char *parmname );

static CPI_PrivateData	 TiffOpenForRead( const CPI_Metadata	 meta,
										  CPI_Metadata 			 imageinfo );

static CPI_Bool			 TiffReadImage( CPI_PrivateData		 handle,
										CPI_Image			*image );
static CPI_Bool			 TiffReadSubImage( CPI_PrivateData	 handle,
										   CPI_Image	  	*image );

static CPI_PrivateData		 TiffOpenForWrite( const CPI_Metadata meta,
											   const CPI_Metadata imageinfo );

static CPI_Bool			 TiffWriteImage( CPI_PrivateData	 handle,
										 CPI_Image			*image );
static CPI_Bool			 TiffWriteSubImage( CPI_PrivateData	 handle,
											CPI_Image	   	*image );

static CPI_Bool			 TiffCloseImage( CPI_PrivateData handle );

// Libtiff Replacement functions
static tsize_t		 tiffReadProc( thandle_t	 fd,
								   tdata_t		 buf,
								   tsize_t		 size );
static tsize_t		 tiffWriteProc( thandle_t	 fd,
									tdata_t		 buf,
									tsize_t		 size );
static toff_t	  	 tiffSeekProc( thandle_t   	 fd,
								   toff_t		 off,
								   int			 whence );
static int			 tiffCloseProc( thandle_t  	 fd );
static void			 tiffError( const char		*module,
								const char		*fmt, 
								va_list 		 ap );
static void			 tiffWarning( const char	*module,
								  const char	*fmt,
								  va_list 		 ap );
static toff_t	  	 tiffSizeProc( thandle_t fd );
void				*_TIFFmalloc( tsize_t		 s );
void				 _TIFFfree( tdata_t p );
void				*_TIFFrealloc( tdata_t p, tsize_t s );
void				 _TIFFmemset( tdata_t p, int v, tsize_t c );
void				 _TIFFmemcpy( tdata_t d, const tdata_t s, tsize_t c );
int					 _TIFFmemcmp( const tdata_t	p1, 
								  const tdata_t	p2, 
								  tsize_t 		c );

// Internal functions
static CPI_Bool			 TiffLoadTiles( TIFF 		*tif, 
										CPI_Image	*image,
										imageState	*state,
										CPI_Uint32	 sizeX,
										CPI_Uint32	 sizeY,
										CPI_Uint32	 offX,
										CPI_Uint32	 offY,
										CPI_Uint16	 bitdepth,
										CPI_Uint16	 channels );
static CPI_Bool			 TiffLoadLines( TIFF		*tif,
										CPI_Image	*image,
										imageState	*state,
										CPI_Uint32	 sizeX,
										CPI_Uint32	 sizeY,
										CPI_Uint32	 offX,
										CPI_Uint32	 offY,
										CPI_Uint16	 bitdepth,
										CPI_Uint16	 channels );
static CPI_Bool			 doMetrics( CPI_Uint16	 photoMetric,
									CPI_Image	*image,
									TIFF		*tif );
