/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	UserLUT
//
// COMMENTS: This demonstrates creating a simple user LUT and
//			 having it show up in the 'Display Conversion' menu
//
*/

#include <stdio.h>
#include <CPI.h>


///////////////////////////////////////
// This is called once on startup. If you were going to read
// in an outside table, you would do it here

static CPI_Bool
DoublerInit( void )
{
	return CPI_TRUE;
}


///////////////////////////////////////
// This cleans up on exit

static CPI_Bool
DoublerShutDown( void )
{
	return CPI_TRUE;
}


///////////////////////////////////////
// this is called for each image viewer update.
// values is the number of color values to convert. 
// For 8bit and 16bit images, you are modifying a collection of
// LUT values. For float, you are modifying the data directly - in 
// other words, this subroutine becomes a function.
// The incoming LUT or pixel values, *src, is modified here and
// returned in *dst. This is because you want your LUT to be
// be part of any upstream color changes, not to replace them.

static void
DoublerCreateLUT( CPI_PrivateData	 handle,
				  CPI_Float32		*dst,
				  const CPI_Float32	*src,
				  CPI_Int32			 values,
				  CPI_Int32			 channels )
{
	values *= channels;

	for ( CPI_Int32	i = 0; i < values; ++i )
	{
		*dst++ = *src++ * 2.0F;
	}
}


////////////////////////////////////
// As with nodes, LUTs are described by a structure like
// this one. This must be present.

static RPI_LUTInfo linfo = {
	{
		"doubler",			// internal name
		"Doubler",			// name which appears in the menu
		"Silicon Grail",    // author
		"v0.1",				// version
		"",					// help URL
		""					// icon name
	},
	DoublerInit,		// init function
	DoublerShutDown,	// shutdown function
	NULL,				// LUT setup
	NULL,				// LUT cleanup
	NULL,				// confidence 
	NULL,				// parm changed
	DoublerCreateLUT,	// create LUT
	NULL				// automatic setup
};


//////////////////////////////////
// Register the plugin. You should use this declaration
// to ensure compatibility across NT, linux and IRIX

CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	return cpiRegisterLUT( &linfo );
}


//////////////////////////////////
// Unregister the plugin - the name to be unregistered must be
// the same as the internal name in the LUTInfo structure

CPIDSOEXPORT void
rpiPluginCleanup( void )
{
	cpiUnregisterLUT( "doubler" );
}

