/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	ChannelSplit
//
// COMMENTS:
//
*/

#include <stdio.h>

#include <CPI.h>


///////////////////////////////////////


static CPI_Bool
ChannelSplitInit( void )
{
	return CPI_TRUE;
}


/////////////////////////////////////


static CPI_Bool
ChannelSplitShutDown( void )
{
	return CPI_TRUE;
}


/////////////////////////////////////


static CPI_Bool
ChannelSplitGetRegion( CPI_ImageContext	*region,
					   CPI_Float32		 myTime,
					   CPI_Uint32		 output,
					   CPI_Bool	 	 	 /*viewerOutput*/ )
{
	CPI_Bool	retval = CPI_FALSE;

	if ( cpiGetInputContext( region, 0, myTime ) && 
		 4 == region->channels )
	{
		if ( output == 0 )
		{
			region->channels	= 3;
			retval 				= CPI_TRUE;
		}
		else if ( output == 1 )
		{
			region->channels 	= 1;
			retval				= CPI_TRUE;
		}
		else
			cpiError( "Channel Split got a request for an invalid output" );
	}
	else
		cpiError( "Channel Split couldn't get input or input didn't have 4 "
				  "channels" );

	return retval;
}


//////////////////////////////////////


static CPI_Bool
ChannelSplitGetErrors( CPI_Float32 time )
{
	CPI_ImageContext 	 input;
	CPI_Bool			 retval	= CPI_FALSE;

	if ( cpiGetInputContext( &input, 0, time ) )
	{
		if ( 4 <= input.channels )
			retval = CPI_TRUE;
		else
			cpiError( "Input must have at least 4 channels!" );
	}
	else
		cpiError( "Couldn't get input context!" );

	return retval;
}


/////////////////////////////////////////


static CPI_ImageOp
ChannelSplitExec( CPI_Float32	time,
				  CPI_Uint8		quality,
				  CPI_Uint32	output,
				  CPI_Bool		/*viewerOutput*/,
				  CPI_Float32	scaleX,
				  CPI_Float32	scaleY )
{
	CPI_ImageOp	retval	= NULL;
	CPI_ImageOp	inputOp	= cpiGetInputOp( 0, time, quality, scaleX, scaleY );

	if ( NULL != inputOp )
	{
		CPI_Metadata	meta	= cpiCreateMetadata();

		if ( output == 0 )
		{
			cpiSetMetaInt32( meta, "channels", 3 );
			retval = cpiAddImageOp( "channelsplit", meta, &inputOp, 1 );
		}
		else if ( output == 1 )
		{
			cpiSetMetaInt32( meta, "channels", 1 );
			retval = cpiAddImageOp( "channelsplit", meta, &inputOp, 1 );
		}

		cpiDeleteMetadata( meta );
	}

	return retval;
}


/////////////////////////////////////


static const char *
ChannelSplitOutputLabel( CPI_Uint32	output )
{
	const char	*retval	= NULL;

	if ( 0 == output )
		retval = "RGB Output";
	else if ( 1 == output )
		retval = "Alpha Output";

	return retval;
}


/////////////////////////////////////


static RPI_NodeInfo ninfo = {
	{
		"channelsplit",
		"Channel Split",
		"Silicon Grail",
		"v1.0"
	},
	ChannelSplitInit,
	ChannelSplitShutDown,
	NULL,
	NULL,
	ChannelSplitGetRegion,
	NULL,
	NULL,
	ChannelSplitGetErrors,
	ChannelSplitExec,
	NULL,
	ChannelSplitOutputLabel,
	NULL,
	NULL,
	1,
	1,
	2,
	2,
	CPI_FALSE
};


///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
// Image Op Functions /////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////


template <class T> static
void split_alpha( CPI_Image	*result,
				  CPI_Image	*input,
				  T			 /*max*/ )
{
	CPI_Uint32	 endX, endY;
	T			*out	= (T *)result->data;
	T			*in		= (T *)input->data;

	endX	= result->myContext.offsetX + result->myContext.sizeX;
	endY	= result->myContext.offsetY + result->myContext.sizeY;

	for ( CPI_Uint32 y = result->myContext.offsetY; y < endY; ++y )
	{
		out = (T *)cpiGetLine( result, y );
		in	= (T *)cpiGetLine( input, y );
		
		for ( CPI_Uint32 x = result->myContext.offsetX; x < endX; ++x )
		{

			in += 3;
			*out++ = *in++;
		}
	}
}


//////////////////////////////////////////


template <class T> static
void split_rgb( CPI_Image	*result,
				CPI_Image	*input,
				T			 /*max*/ )
{
	CPI_Uint32	 endX, endY;
	T			*out	= (T *)result->data;
	T			*in		= (T *)input->data;
	CPI_Uint32	 skip	= ( input->myContext.channels - 3 );

	endX	= result->myContext.offsetX + result->myContext.sizeX;
	endY	= result->myContext.offsetY + result->myContext.sizeY;

	for ( CPI_Uint32 y = result->myContext.offsetY; y < endY; ++y )
	{
		out = (T *)cpiGetLine( result, y );
		in	= (T *)cpiGetLine( input, y );
		
		for ( CPI_Uint32 x = result->myContext.offsetX; x < endX; ++x )
		{
			*out++ = *in++;
			*out++ = *in++;
			*out++ = *in++;
			in += skip ;
		}
	}
}


/////////////////////////////////////////////


static CPI_Bool
ChannelSplitOutputRegion( CPI_ImageContext	*output,
						  CPI_ImageContext	 inputs[],
						  CPI_Uint32		 numInputs,
						  CPI_Metadata		 parms )
{
	CPI_Bool	retval = CPI_FALSE;

	if ( numInputs > 0 )
	{
		*output = inputs[0];

		if ( cpiGetMetaInt32( parms, "channels", &output->channels ) )
			retval = CPI_TRUE;
		else
			cpiError( "Couldn't get channels parameter from metadata" );
	}

	return retval;
}


/////////////////////////////////////////////


static CPI_Bool
ChannelSplitProcess( CPI_Image		*result,
					 CPI_Image	 	 inputs[],
					 CPI_Uint32	 	 numInputs,
					 CPI_Metadata	 myParms )
{
	CPI_Bool	retval = CPI_FALSE;

	if ( NULL != result && numInputs == 1 )
	{
		CPI_Int32		 	channels;
		CPI_ImageContext	c = inputs[0].myContext;

		retval = cpiGetMetaInt32( myParms, "channels", &channels );

		if ( retval )
		{
			if ( channels == 1 )
			{
				if ( c.bitsPerPel == 8 )
					split_alpha( result, &inputs[0], (CPI_Uint8)255 );
				else if ( c.bitsPerPel == 16 )
					split_alpha( result, &inputs[0], (CPI_Uint16)65535 );
				else if ( c.bitsPerPel == 32 )
					split_alpha( result, &inputs[0], (CPI_Float32)1.0 );
				else
				{
					cpiError( "Inputs image has an invalid bit depth!" );
					retval = CPI_FALSE;
				}
			}
			else if ( channels == 3 )
			{
				if ( c.bitsPerPel == 8 )
					split_rgb( result, &inputs[0], (CPI_Uint8)255 );
				else if ( c.bitsPerPel == 16 )
					split_rgb( result, &inputs[0], (CPI_Uint16)65535 );
				else if ( c.bitsPerPel == 32 )
					split_rgb( result, &inputs[0], (CPI_Float32)1.0 );
				else
				{
					cpiError( "Inputs image has an invalid bit depth!" );
					retval = CPI_FALSE;
				}	
			}
			else
			{
				cpiError( "Invalid channels value!" );
				retval = CPI_FALSE;
			}
		}

	}

	return retval;
}


///////////////////////////////////


static RPI_ImageOpInfo opinfo =
{
	{
		"channelsplit",
		"Channel Split",
		"Silicon Grail",
		"1.0"
	},
	ChannelSplitProcess,
	NULL,
	ChannelSplitOutputRegion,
	NULL,
	NULL,
	NULL,
	1,
	CPI_FALSE,
	CPI_FALSE
};
	

//////////////////////////////////////


CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	CPI_Bool	retval = CPI_TRUE;

	retval = cpiRegisterImageOp( &opinfo );

	if ( retval )
		retval = cpiRegisterNode( &ninfo );

	return retval;
}


///////////////////////////////////////


CPIDSOEXPORT void
upiPluginCleanup( void )
{
	cpiUnregisterImageOp( "channelsplit" );
	cpiUnregisterNode( "channelsplit" );
}
