/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	Pizza
//
// COMMENTS: Select and order a pizza via email
//
*/

#include <stdio.h>
#include <math.h>
#include <CPI.h>

#include "Pizza.h"	// created via the Gimp

///////////////////////////////////////////

static const char *sizemenu[] =
{
	"small",
	"medium",
	"large",
	"slice",
	NULL
};

static const char *crustmenu[] =
{
	"thin",
	"thick",
	NULL
};

static struct topping
{
	const char 	*code;
	const char 	*label;
	CPI_Bool	 wantIt;
} toppings[] =
{
	{ "pep",	"Pepperoni",		CPI_FALSE },
	{ "mush",	"Mushrooms",		CPI_FALSE },
	{ "gpep",	"Green Peppers",	CPI_FALSE },
	{ "saus",	"Sausage",			CPI_FALSE },
	{ "olive",	"Olives",			CPI_FALSE },
	{ "xchese",	"Extra Cheese",		CPI_FALSE },
};

#define NTOPPINGS (CPI_Int32)( sizeof(toppings) / sizeof(struct topping) )

static void
submit( const char 		*/*nodename*/,
		const char 		*/*parmname*/,
		CPI_Float32		 time )
{
	CPI_Int32	i, size, crust;
	char		strval[100], name[50];
	CPI_Bool	oneOut = CPI_FALSE;

	cpiGetString( strval, 100, "order", time );
	cpiGetInteger( &size, "size", time );
	cpiGetInteger( &crust, "crust", time );
	for ( i = 0; i < NTOPPINGS; i++ )
	{
		sprintf( name, "toppings.%s", toppings[i].code );
		cpiGetInteger( &toppings[i].wantIt, name, time );
	}

	printf("Order to: %s\n", strval);
	printf("Please deliver a %s, %s crust pizza\n", sizemenu[size], 
			crustmenu[crust]);
	printf("Extra Toppings:\n");
	for (i = 0; i < NTOPPINGS; i++ )
	{
		if ( toppings[i].wantIt == CPI_TRUE )
		{
			printf("\t%s\n", toppings[i].label );
			oneOut = CPI_TRUE;
		}
	}

	if ( oneOut == CPI_FALSE )
		printf("\tNone\n");
}


static CPI_Bool
PizzaInit( void )
{
	CPI_Int32	i;
	char		helpstring[128];

	cpiAddString( "order", "Send to", "orders@bigfatpizza.com",
				  CPI_FALSE, CPI_PARM_DEFAULT,
				  "Email address to send order to", NULL );

	cpiAddMenu( "size", "Size", 0, sizemenu,
				"Choose the size of the pizza", NULL );

	cpiAddMenu( "crust", "Crust", 0, crustmenu,
				"Choose the type of crust", NULL );

	cpiStartGroup( "toppings", "Toppings", CPI_PARM_DEFAULT,
				   "This folder holds choices for toppings", NULL );
	for ( i = 0; i < NTOPPINGS; i++ )
	{
		sprintf( helpstring,  "Add %s to the pizza", toppings[i].label );
		cpiAddToggle( toppings[i].code, toppings[i].label, 
					  CPI_FALSE, CPI_PARM_DEFAULT, helpstring, NULL );
	}
	cpiEndGroup( );

	cpiAddButton( "submit", "Send Order", submit,
				  CPI_PARM_DEFAULT,
				  "Send email ordering the pizza", NULL );

	// set up the pizza icon
	cpiRegisterIconData( "Pizza.tiff", 60, 45, icon_image, 
						 255, 0, 255 );

	// define the location of the pizza icon
	cpiDefineIcon( "Nodes/Pizza", "Pizza.tiff",
				   NULL, NULL, NULL, NULL, NULL, 255, 0, 255 );

	return CPI_TRUE;
}


///////////////////////////////////////////


CPI_Bool
PizzaShutDown( void )
{
	return CPI_TRUE;
}



//////////////////////////////////////////


CPI_ImageOp
PizzaExec( CPI_Float32	myTime,
		   CPI_Uint8	quality,
		   CPI_Uint32	/*output*/,
		   CPI_Bool		/*viewerOutput*/,
		   CPI_Float32	scaleX,
		   CPI_Float32	scaleY )
{
	CPI_ImageOp	 retval	= NULL;
	CPI_ImageOp	 myInputs;

	myInputs = cpiGetInputOp( 0, myTime, quality, scaleX, scaleY );
	if ( myInputs == NULL )
		return NULL;

	retval = cpiAddImageOp( "pizza", NULL, &myInputs, 1 );
		
	return retval;
}


////////////////////////////////////////////

static RPI_NodeInfo ninfo = {
	{
		"pizza",
		"Pizza",
		"Silicon Grail",
		"1.0", NULL, NULL
	},
	PizzaInit,
	PizzaShutDown,
	NULL,
	NULL,		// InputChanged
	NULL,
    NULL,
	NULL,		// PizzaGetRange,
	NULL,
	PizzaExec,
	NULL,		// InputLabel,
	NULL,		// OutputLabel,
	NULL,		// ViewerOutputLabel
	NULL,		// NumViewerOutputs
	1, 
	1,
	1,
	1,
	CPI_FALSE
};


///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
// Image Op Functions /////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////


template <class T>
static
void copy_image( CPI_Image		&input,
				 CPI_Image		*result, 
				 T 			 	 /*max*/ )
{
	CPI_Uint32	 i, x, y;
	CPI_Uint32	 endX, endY, chans;
	T			*pin, *pout;

	endX	= result->myContext.offsetX + result->myContext.sizeX;
	endY	= result->myContext.offsetY + result->myContext.sizeY;
	chans	= result->myContext.channels;
	
	for ( y = result->myContext.offsetY; y < endY; y++ )
	{
		pin  = (T *)cpiGetLine( &input, y );
		pout = (T *)cpiGetLine( result, y );
		for ( x = result->myContext.offsetX; x < endX; x++ )
		{
			for ( i = 0; i < chans; i++ )
			{
				*pout = *pin;
				pin++;
				pout++;
			}
		}
	}
}


/////////////////////////////////////////


CPI_Bool
PizzaProcess( CPI_Image		*result,
			  CPI_Image	 	 inputs[],
			  CPI_Uint32	 /*numInputs*/,
			  CPI_Metadata	 /*myParms*/ )
{
	CPI_Bool	retval	= CPI_TRUE;

	if ( result->myContext.bitsPerPel == 8 )
		copy_image( inputs[0], result, (CPI_Uint8)255 );
	else if ( result->myContext.bitsPerPel == 16 )
		copy_image( inputs[0], result, (CPI_Uint16)65535 );
	else if ( result->myContext.bitsPerPel == 32 )
		copy_image( inputs[0], result, (CPI_Float32)1.0F );
	else
	{
		cpiError( "Result image has an invalid bit depth!" );
		retval = CPI_FALSE;
	}
		

	return retval;
}


/////////////////////////////////////


static RPI_ImageOpInfo opinfo = 
{
	{
		"pizza",
		"Pizza",
		"Silicon Grail",
		"1.0", NULL, NULL
	},
	PizzaProcess,
	NULL,			// PizzaInputRegion
	NULL,			// OutputRegion
	NULL,			// ControlProcessing
	NULL,			// PizzaSetupProcessing
	NULL,			// PizzaCleanupProcessing
	1,
	CPI_FALSE,
	CPI_FALSE,
};


//////////////////////////////////////////


CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	CPI_Bool retval = true;

	retval =  cpiRegisterImageOp( &opinfo );

	if ( retval )
		retval = cpiRegisterNode( &ninfo );

	return retval;	
}


//////////////////////////////////////////////


CPIDSOEXPORT void
upiPluginCleanup( void )
{
	cpiUnregisterImageOp( "pizza" );
	cpiUnregisterNode( "pizza" ); 
}
