/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:    Mask185
//
// COMMENTS: This demonstrates an overlay which is not attached
//           to any node, but which instead is attached to the
//           image viewer(s). After installing this plugin, you
//           will see the menu name appear on the right mouse 
//           menu in any image viewer. The overlay is a 1:1.85
//			 ratio black mask for the viewer.
//
*/

#include <stdio.h>
#include <math.h>
#include <CPI.h>

//////////////////////////////////////
// This plugin is so simple, we don't even need Create & Destroy 
// functions.


///////////////////////////////////////
// Do need to define the render, which is trivial

static void
OverlayRender( CPI_PrivateData handle, CPI_Bool isPicking )
{
	CPI_Float32		width = (1.0 / 1.85);
	CPI_Float32		offset = (width / 2.0);

	cpiSetColor( 0.0, 0.0, 0.0 );

	// top cutoff
	cpiDrawFilledRect( 0.0, 0.0, 1.0, offset );
	// bottom cutoff
	cpiDrawFilledRect( 0.0, (offset+width), 1.0, 1.0 );

}


////////////////////////////////////////
// As with nodes and image ops, overlays are defined
// to Rayz with a structure. This structure must be
// present, but the routines it defines are optional.
// Missing routines must be marked with NULL

static RPI_OverlayInfo oinfo = {
	{
		"mask185",			// internal name
		"Mask 1:1.85",		// menu name
		"Silicon Grail",
		"v1.0"
	},
	NULL,		// Init
	NULL,		// Shutdown
	NULL,		// CreateInstance
	NULL,		// DestroyInstance
	NULL,		// SourceChanged
	NULL,		// FullSizeChanged
	NULL,		// SourceOutOfDate
	OverlayRender,
	NULL,		// OverlayMouseDown
	NULL,		// OverlayLeftMouseDrag
	NULL,		// OverlayLeftMouseUp
	NULL,		// OverlayMouseEnter
	NULL,		// OverlayMouseDrag
	NULL,		// OverlayMouseExit
	NULL,
	// since the node name is NULL, the overlay is
	// attached directly to any image viewers that exist
	CPI_FALSE
};


////////////////////////////////////
// register this plugin

CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	CPI_Bool retval = CPI_TRUE;

	retval =  cpiRegisterOverlay( &oinfo );

	return retval;	
}


//////////////////////////////////////////////
// unregister this plugin. Note the internal name is used.

CPIDSOEXPORT void
upiPluginCleanup( void )
{
	cpiUnregisterOverlay( "mask185" );
}
