/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 N. Seward
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:    SimpleOverlay
//
// COMMENTS: Demonstrates an image overlay (one not tied to any
//			 node). This creates a simple field chart.
//
*/

#include <stdio.h>
#include <math.h>
#include <CPI.h>

#define LEFT	0

///////////////////////////////////////
// Need to define the render routine. See include/Overlay.h
// for a list of drawing routines.

void
OverlayRender( CPI_PrivateData handle, CPI_Bool isPicking )
{
	// outline of image
	cpiSetColor( 1.0, 1.0, 1.0 );
	cpiDrawRect( 0.0, 0.0, 1.0, 1.0 );
	cpiDrawText( LEFT, 0.05, 0.0, 0.1, 0.025, "CAMERA");
	cpiDrawText( LEFT, 0.85, 0.975, 0.1, 0.025, "CAMERA");

	// 1.85 lines
	cpiDrawRect( 0.023, 0.1475, 0.954, 0.705 );
	cpiDrawText( LEFT, 0.31, 0.1445, 0.1, 0.025, "1.85");
	cpiDrawText( LEFT, 0.63, (1.0 - 0.1445 - 0.025), 0.1, 0.025, "1.85");

	// crosshairs
	cpiDrawLine( 0.0, 0.5, 1.0, 0.5 );
	cpiDrawLine( 0.5, 0.0, 0.5, 1.0 );
	cpiDrawCircle( 0.5, 0.5, 0.0218 );

}


////////////////////////////////////////
// As with nodes and image ops, overlays are defined
// to Rayz with a structure. This structure must be
// present, but the routines it defines are optional.
// Missing routines must be marked with NULL

static RPI_OverlayInfo oinfo = {
	{
		"simplechart",		// internal name
		"Image Chart",		// menu name
		"Silicon Grail",
		"v0.1"
	},
	NULL,		// Init
	NULL,		// Shutdown
	NULL,		// CreateInstance
	NULL,		// DestroyInstance
	NULL,		// OverlaySourceChanged
	NULL,		// OverlayFullSizeChanged
	NULL,		// OverlaySourceOutOfDate
	OverlayRender,
	NULL,		// OverlayMouseDown
	NULL,		// OverlayLeftMouseDrag
	NULL,		// OverlayLeftMouseUp
	NULL,		// OverlayMouseEnter
	NULL,		// OverlayMouseDrag
	NULL,		// OverlayMouseExit
	NULL,
};


////////////////////////////////////
// register this plugin

CPIDSOEXPORT CPI_Bool
rpiPluginInit( void )
{
	CPI_Bool retval = CPI_TRUE;

	retval =  cpiRegisterOverlay( &oinfo );

	return retval;	
}


//////////////////////////////////////////////
// unregister this plugin

CPIDSOEXPORT void
upiPluginCleanup( void )
{
	cpiUnregisterOverlay( "simplechart" );
}
