/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	CPI_CurveIOProvider
//
// COMMENTS:
//
*/


#ifndef _CPI_CurveIOProvider_h_
#define _CPI_CurveIOProvider_h_

#include <CPI/CPI_Base.h>
#include <CPI/CPI_CurveID.h>


#ifdef __cplusplus
extern "C" {
#endif


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the plugin was initialized
//		 	   successfully.
*/
typedef CPI_Bool	(*RPI_CurveIOInitFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the plugin was shut down sucessfully
*/
typedef CPI_Bool	(*UPI_CurveIOShutDownFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	const CPI_Metadata meta - This is the metadata which contains all of the
//							  information about the file which you are going
//							  to attempt to open such as filename, file access
//							  mode, etc.  Usually just passed into a cpiOpen()
//	CPI_CurveIDList curves - This is a list of curves which you are able to
//							 import from this file.  Add all of the curves
//							 (by name, which will then be passed into your
//							 import function) which you can import.
// Return Value:
//	CPI_PrivateData - This user defined structure is opaque to rayz, but should
//					  contain any information which will be needed when reading
//					  the curve data.  If opening the file failed or you
//					  cannot import curves from this file, return NULL.
// Undefined Behavior:
//	If this function is not provided then rayz will assume that your plugin
//	cannot import curves.  This is useful for formats which you want to be
//	able to export to, but not import from.
*/
typedef CPI_PrivateData	(*UPI_CurveIOOpenForImportFn)(
	const CPI_Metadata	 meta,
	CPI_CurveIDList		 curves );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData handle - User defined structure containing format specific
//							 information.
//	const char *curveName - This is the name of the curve you being asked to
//							import.
//
// Return Value:
//	char * - a string which should no longer be referenced upon return, and
//			 will be freed by the consumer, representing the new curve
//			 represented as a rayz curve string.
*/
typedef char *(*UPI_CurveIOImportFn)( CPI_PrivateData	 handle,
									  const char		*curveName );


/*
//////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	const CPI_Metadata meta - This is the metadata which contains all of the
//							  information about the file which you are going
//							  to attempt to open such as filename, file access
//							  mode, etc.  Usually just passed into a cpiOpen()
//	const CPI_CurveIDList curves - This is a list of curves which you will be
//								   asked to export to this file.
// Return Value:
//	CPI_PrivateData - This user defined structure is opaque to rayz, but should
//					  contain any information which will be needed when reading
//					  the curve data.  If opening the file failed, return
//					  NULL.
// Undefined Behavior:
//	If this function is not provided then rayz will assume that your plugin
//	cannot export curves.  This is useful for formats which you want to be
//	able to import from, but not export to.
*/
typedef CPI_PrivateData	(*UPI_CurveIOOpenForExportFn)(
	const CPI_Metadata		 meta,
    const CPI_CurveIDList	 curves );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData handle - User defined structure containing format specific
//							 information.
//	const char *curveName - This is the name of the curve you are currently
//							being asked to export.
//	const char *curveString - This is the curve as rayz would save it. This
//							  function should be able to read and convert
//							  this string to whatever format is desired.
// Return Value:
//	CPI_Bool - This function should return CPI_TRUE if the curve was
//			   successfully exported.
*/
typedef CPI_Bool	(*UPI_CurveIOExportFn)( CPI_PrivateData	 handle,
											const char		*curveName,
											const char		*curveString );

/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData handle - User Defined structure containing format specific
//							 information.  NOTE: If cpiAlloc'ed in
//							 openForExport or openForImport this should
//							 be free'd here
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the file was closed successfully
*/
typedef CPI_Bool		 (*UPI_CurveIOCloseFn)( CPI_PrivateData	 handle );


/*
// For any optional (upi labeled) functions you don't wish to provide,
// fill in NULL
*/
typedef struct
{
	RPI_PluginInfo				 rpiPluginInfo;

	RPI_CurveIOInitFn			 rpiCurveIOInit;
	UPI_CurveIOShutDownFn		 rpiCurveIOShutDown;

	UPI_CurveIOOpenForImportFn	 upiCurveIOOpenForImport;
	UPI_CurveIOImportFn			 upiCurveIOImport;

	UPI_CurveIOOpenForExportFn	 upiCurveIOOpenForExport;
	UPI_CurveIOExportFn			 upiCurveIOExport;

	UPI_CurveIOCloseFn			 upiCurveIOClose;

	/* Information about the image format */
	const char					*upiExtensions;
} RPI_CurveIOInfo;

#ifdef __cplusplus
}
#endif

#endif /* _CPI_CurveIOProvider_h_ */


