/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	CPI_ExpressionProvider
//
// COMMENTS:
//
*/

#ifndef _CPI_ExpressionProvider_h_
#define _CPI_ExpressionProvider_h_

#include <CPI/CPI_Base.h>


#ifdef __cplusplus
extern "C" {
#endif


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the plugin was initialized
//		 	   successfully.
*/
typedef CPI_Bool	(*RPI_ExpressionInitFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the plugin was shut down sucessfully
*/
typedef CPI_Bool	(*UPI_ExpressionShutDownFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//  const CPI_Float32 args[] - This is an array of the arguments which the
//							   user has passed to your math function.
//	CPI_Uint32 numArgs - Number of arguments in the args[] array
//	CPI_Float32	*result - Place the resulting value here
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the function was applied correctly
//			   and the value placed in the *result ptr
*/
typedef CPI_Bool	(*RPI_ExpressionApplyFn)( const CPI_Float32	 args[],
											  CPI_Uint32 		 numArgs,
											  CPI_Float32		*result );

/*
// For any optional (upi labeled) functions you don't wish to provide,
// fill in NULL
*/
typedef struct
{
	RPI_PluginInfo				 rpiPluginInfo;

	RPI_ExpressionInitFn		 rpiExpressionInit;
	UPI_ExpressionShutDownFn	 upiExpressionShutDown;

	RPI_ExpressionApplyFn		 rpiExpressionApply;

	const char					*rpiHelpString;
	CPI_Uint32					 rpiMinimumArguments;
	CPI_Uint32					 rpiMaximumArguments;
} RPI_ExpressionInfo;

#ifdef __cplusplus
}
#endif

#endif /* _CPI_ExpressionProvider_h_ */
