/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
*/

#ifndef _CPI_FileID_h_
#define _CPI_FileID_h_

#include <CPI/CPI_Types.h>

#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************************************
 ** File ID list utility functions
 **
 ** A File ID is a generic string descriptor for a file.
 ** It is intended to be generated by a file manager for use in places
 ** like Disk Input, where you can represent many files with one string
 **
 ******************************************************************************/


CPIDSOIMPORT CPI_FileIDList	 cpiCreateFileIDList( void );
CPIDSOIMPORT void			 cpiDeleteFileIDList( CPI_FileIDList idlist );

CPIDSOIMPORT CPI_Uint32		 cpiGetFileIDListSize( CPI_FileIDList idlist );

// fileid can be temporary storage - a copy will be made of it
CPIDSOIMPORT void			 cpiAddFileIDToList( CPI_FileIDList	 idlist,
												 const char		*fileid );

CPIDSOIMPORT const char		*cpiGetFileIDFromList( CPI_FileIDList	 idlist,
												   CPI_Uint32		 idx );

/*
 * If sortfn is NULL, will use the default alpha-numerical sort aka strcmp
 */
CPIDSOIMPORT void			 cpiSortFileIDList( CPI_FileIDList		 idlist,
												CPI_SortFn			 sortfn );


/*******************************************************************************
 ** File ID Sequence list utility functions
 **
 ** Given the above file lists, compress many files into one descriptor for
 ** those files, with associated information about the group of files
 **
 ******************************************************************************/


CPIDSOIMPORT CPI_FileIDSeqList	 cpiCreateFileIDSeqList( void );
CPIDSOIMPORT void				 cpiDeleteFileIDSeqList( CPI_FileIDSeqList seq );

CPIDSOIMPORT CPI_Uint32			 cpiGetFileIDSeqListSize( CPI_FileIDSeqList seq );

/*
 * fileid can be temporary storage - a copy will be made of it, but for
 * speed reasons, the metadata should be released to the FileIDSeqList,
 * not to be freed...
 */
CPIDSOIMPORT void				 cpiAddFileIDToSeqList( CPI_FileIDSeqList	 idlist,
														const char			*fileid,
														CPI_Metadata		 metadata );

CPIDSOIMPORT const char			*cpiGetFileIDFromSeqList( CPI_FileIDSeqList	 idlist,
														  CPI_Uint32		 idx );
CPIDSOIMPORT CPI_Metadata		 cpiGetMetadataFromSeqList( CPI_FileIDSeqList	 idlist,
															CPI_Uint32			 idx );

/*
 * If sortfn is NULL, will use the default alpha-numerical sort, aka strcmp
 */
CPIDSOIMPORT void				 cpiSortFileIDSeqList( CPI_FileIDSeqList	 idlist,
													   CPI_SortFn			 sortfn );


/*
 * seqlist will be filled with sequences construed from the file IDs in fids.
 * Anywhere there is a sequence will be filled with some variant of
 * $F: $F, ${F}, $F2, ${F2}, $F3, ${F3}, $F4, ${F4}....
 * where sequences are found by looking for numbers in the string.
 *
 * NB: If there is no sequence in the file, the metadata in the
 *     CPI_FileIDSeqList will be NULL for that entry
 */
CPIDSOIMPORT CPI_Bool		 cpiDefaultSeqSearch( CPI_FileIDSeqList		 seqlist,
												  const CPI_FileIDList	 fids );

CPIDSOIMPORT CPI_Bool		 cpiDefaultGlobalVarExpand( const char	*oldFileID,
														char		*newFileID,
														CPI_Int32    buffersize );

CPIDSOIMPORT CPI_Bool	  	 cpiDefaultFrameVarExpand( const char	*oldFileID,
													   char			*newFileID,
													   CPI_Int32	 buffersize,
													   CPI_Float32	 curSample );


#ifdef __cplusplus
}
#endif

#endif /* _CPI_FileID_h_ */

