/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
*/

#ifndef _CPI_ImageProvider_h_
#define _CPI_ImageProvider_h_

#include <CPI/CPI_Base.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
// These flags can be OR'ed together to help define the supported features
// of a particular image reader
*/
#define CPI_IMAGE_RGB					0x0001
#define CPI_IMAGE_RGBA					0x0002
#define CPI_IMAGE_UINT8					0x0004
#define CPI_IMAGE_UINT16				0x0008
#define CPI_IMAGE_FLOAT32				0x0010
#define CPI_IMAGE_PLANAR				0x0020
#define CPI_IMAGE_REVERSE				0x0040
#define CPI_IMAGE_MOTION				0x0080
#define CPI_IMAGE_READFULL				0x0100
#define CPI_IMAGE_WRITEFULL				0x0200
#define CPI_IMAGE_LOG					0x0400
#define CPI_IMAGE_LOG_ONLY				0x0800
#define CPI_IMAGE_EASY_TO_DETERMINE		0x1000
#define CPI_IMAGE_HARD_TO_DETERMINE		0x2000
#define CPI_IMAGE_CANNOT_DETERMINE		0x4000

/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Bool initforread - This function gets called twice, once to initialize
//						   the parameters for imagein and again for imageout.
//						   if 'initforread' is true then we are working on
//						   imagein, if false then imageout.
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the image reader was successfully
//			   initialized.
*/
typedef CPI_Bool		 (*RPI_ImageInitFn)( CPI_Bool initforread );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the image reader was successfullly
//			   shutdown.
*/
typedef CPI_Bool		 (*RPI_ImageShutDownFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	const char *parmname - This is the name of the parm for which the value has
//						   changed.
// Return Value:
//	void
*/
typedef void			 (*UPI_ParmChangedFn)( const char *parmname );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	const CPI_Metadata meta - This is the metadata which contains all of the
//							  information about the file which you are going
//							  to attempt to open such as filename, file access
//							  mode, etc.  Usually just passed into a cpiOpen()
//	CPI_Metadata imageinfo - This metadata needs to be filled up with
//							 information about the current image you are
//							 opening ( ex: Width, Height, Channels, Bitdepth )
//
//						   REQUIRED: The fields that are required are
//							   Full Size x, y
//							   Channels
//							   BitDepth
//
//						   OPTIONAL: Optional fields are
//							   Offset x, y
//							   Size x, y	   - Mostly used for formats like
//												 RLL where there is an active
//												 area inside a larger size
//
// Return Value :
//	CPI_PrivateData - This user defined structure is opaque to rayz, but should
//					  contain any information which will be needed when reading
//					  the image data.  If opening the image failed, return
//					  NULL.
// Undefined Behavior:
//	If this function is not provided then rayz will assume that your format
//	cannot read images, this is useful for formats which your want to be able
//	to write, but not read back into rayz.
*/
typedef CPI_PrivateData	(*UPI_OpenForReadFn)( const CPI_Metadata	meta,
											  CPI_Metadata			imageinfo );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData handle - User defined structure containing format specific
//							 information.
//	CPI_Image *image - This is the image to read the file into.  This function
//					   should ALWAYS receive a full image ( ie, full size and
//					   no offset )
// Return Value:
//	CPI_Bool - Should return CPI_TRUE if the image was read successfully
//
// Undefined Behavior:
//  If this function is not provided then rayz will try to call readSubImage()
//  with the full size and no offsets.
*/
typedef CPI_Bool		 (*UPI_ReadImageFn)( CPI_PrivateData	 handle,
											 CPI_Image			*image );
/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData handle - User defined structure containing format specific
//							 information.
//	CPI_Image *image - This is the image to read the file into.  This function
//					   will receive images containing non-full sizes and can
//					   can contain offsets.
//
// Return Value:
//	CPI_Bool - Should return CPI_TRUE if the sub-image was read successfully
//
// Undefined Behavior:
//	If this function is not provided then rayz will call readFullImage() and
//	crop out the region it needs (slower)
*/
typedef CPI_Bool		 (*UPI_ReadSubImageFn)( CPI_PrivateData	 handle,
												CPI_Image		*image );

/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	const CPI_Metadata meta - This is the metadata which contains all of the
//							  information about the file which you are going
//							  to attempt to open such as filename, file access
//							  mode, etc.  Usually just passed into a cpiOpen()
//	const CPI_Metadata imageinfo - This metadata contains all of the info
//								   about the image you are going to be writing.
//								   Such as Height, Width, Channels, Bitdepth
//
// Return Value:
//	CPI_PrivateData - User Defined structure containing format specific info.
//					  Return NULL if you cannot write this image.
//
// Undefined Behavior - If this function is undefined then rayz will assume
//						that writing is unavailable for this format.  Useful
//						for making read-only formats.
*/
typedef CPI_PrivateData	 (*UPI_OpenForWriteFn)( const CPI_Metadata meta,
												const CPI_Metadata imageinfo );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData handle - User Defined structure containing format specific
//							 information
//	CPI_Image *image - Image which should be written out to disk.  This
//					   function should ALWAYS recieve a full image (ie full
//					   size and no offsets )
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the image was written successfully
//
// Undefined Behavior - If this function is undefined rayz will try to call
//						writeSubImage() with no offsets and a full size
*/
typedef CPI_Bool		 (*UPI_WriteImageFn)( CPI_PrivateData		 handle,
											  CPI_Image				*image );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData handle - User Defined structure containing format specific
//							 information
//	CPI_Image *image - This is the image to read the file into.  This function
//					   will receive images containing non-full sizes and can
//					   can contain offsets.
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the sub-image was written successfully
*/
typedef CPI_Bool		 (*UPI_WriteSubImageFn)( CPI_PrivateData	 handle,
												 CPI_Image			*image );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData handle - User Defined structure containing format specific
//							 information.  NOTE: If cpiAlloc'ed in openForRead,
//							 or openForWrite this should be free'd here
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the file was closed successfully
*/
typedef CPI_Bool		 (*UPI_CloseImageFn)( CPI_PrivateData	 handle );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData handle - User Defined structure containing format specific
//							 information.
//	CPI_Uint32 framenum - Frame number that has been requested
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if we were successfully able to seek to
//			   the requested frame
//
// Undefined Behavior - This function should only be provided for formats which
//						define the CPI_IMAGE_MOTION flag.  It's used for
//						formats which store mutliple frames in 1 file.  If not
//						provided, any motion file will only be able to read its
//						first frame.
*/
typedef CPI_Bool		 (*UPI_FrameSeekFn)( CPI_PrivateData	 handle,
											 CPI_Uint32		 framenum );

/*
// For any optional (upi labeled) functions you don't wish to provide,
// fill in NULL
*/
typedef struct
{
	RPI_PluginInfo		 rpiPluginInfo;

	RPI_ImageInitFn		 rpiImageInit;
	RPI_ImageShutDownFn	 rpiImageShutdown;
	UPI_ParmChangedFn	 upiParmChanged;
	UPI_OpenForReadFn	 upiOpenForRead;
	UPI_ReadImageFn		 upiReadImage;
	UPI_ReadSubImageFn	 upiReadSubImage;
	UPI_OpenForWriteFn	 upiOpenForWrite;
	UPI_WriteImageFn	 upiWriteImage;
	UPI_WriteSubImageFn	 upiWriteSubImage;
	UPI_CloseImageFn	 upiCloseImage;
	UPI_FrameSeekFn		 upiFrameSeek;

	/* Information about the image format */
	CPI_Uint32			 rpiCaps;
	const char			*upiExtensions;
} RPI_ImageInfo;

#ifdef __cplusplus
}
#endif

#endif /* _CPI_ImageProvider_h_ */

