/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	CPI_LUTProvider
//
// COMMENTS:
//
*/

#ifndef _CPI_LUTProvider_h_
#define _CPI_LUTProvider_h_

#include <CPI/CPI_Base.h>


#ifdef __cplusplus
extern "C" {
#endif


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the LUT was initialized successfully
*/
typedef CPI_Bool	(*RPI_LUTInitFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the LUT was shut down successfully
*/
typedef CPI_Bool	(*UPI_LUTShutDownFn)( void );



/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_ImageContext - This contains the contents of the image which is
//					   currently in the image viewer
// Return Value:
//	CPI_PrivateData	- This is an opaque (to rayz) pointer which will be passed
//					  into all of the LUT functions.  This should allow you to
//					  carry around any neccessary information about a LUT.
*/
typedef CPI_PrivateData	(*UPI_LUTSetupFn)( CPI_ImageContext	context );



/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//  CPI_PrivateData	- This is an opaque (to rayz) pointer which will be passed
//					  into all of the LUT functions.  This should allow you to
//					  carry around any neccessary information about a LUT.
// Return Value:
//	None
*/
typedef void (*UPI_LUTCleanupFn)( CPI_PrivateData	 handle );



/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_ImageContext - This contains the contents of the image which is
//					   currently in the image viewer
// Return Value:
//	CPI_Uint8 - This should be a value from 0 - 100 which represents your
//				confidence in correctly displaying the current image.  For
//				example : A cineon lookup table will return a much higher value
//				for a log image than it would for a linear one..
*/
typedef CPI_Uint8	(*UPI_LUTConfidenceFn)( CPI_ImageContext context );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	const char *parmname - the name of the parm whose value has been changed.
//						   This gets called with a NULL parmname every time
//						   the node is added.
// Return Value:
//	None
*/
typedef void		(*UPI_LUTParmChangedFn)( const char	*parmname );



/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData - This is an opaque (to rayz) pointer which will be passed
//					  into all of the LUT functions.  This should allow you to
//					  carry around any neccessary information about a LUT.
//	CPI_Float32	*dst - Array of float values which needs to be filled out
//	const CPI_Float32 *src - Array of the original float values
//	CPI_Int32 pixels - Number of pixels in the two arrays
//	CPI_Int32 channels - Number of channels each pixel contains
//
// Return Value:
//	void
*/
typedef void	(*RPI_LUTCreateLUTFn)( CPI_PrivateData		 handle,
									   CPI_Float32			*dst,
									   const CPI_Float32	*src,
									   CPI_Int32			 pixels,
									   CPI_Int32			 channels );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_PrivateData - This is an opaque (to rayz) pointer which will be passed
//					  into all of the LUT functions.  This should allow you to
//					  carry around any neccessary information about a LUT.
//
//	CPI_ImageContext - This contains the contents of the image which is
//					   currently in the image viewer
//
// Return Value:
//	void
//
// Comments:
//  Gives the plugin writer a chance to change current values to match what is
// in the image context. If this is NULL, it will default to resetting everything
// to the default values
//
*/
typedef void	(*UPI_LUTAutomaticSetup)( CPI_PrivateData	 handle,
										  CPI_ImageContext	 context );


typedef struct
{
	RPI_PluginInfo			rpiPluginInfo;

	RPI_LUTInitFn			rpiLUTInit;
	UPI_LUTShutDownFn		upiLUTShutDown;

	UPI_LUTSetupFn			upiLUTSetup;
	UPI_LUTCleanupFn		upiLUTCleanup;

	UPI_LUTConfidenceFn		upiLUTConfidence;

	UPI_LUTParmChangedFn	upiLUTParmChanged;
	RPI_LUTCreateLUTFn		rpiLUTCreate;

	UPI_LUTAutomaticSetup	upiLUTAutomaticSetup;
} RPI_LUTInfo;


#ifdef __cplusplus
}
#endif

#endif /* _CPI_LUTProvider_h_ */
