/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
*/

#ifndef _CPI_Metadata_h_
#define _CPI_Metadata_h_

#include <CPI/CPI_Types.h>

#ifdef __cplusplus
extern "C" {
#endif


/* //////////////////////////////////////// */


/* Metadata Tools */
CPIDSOIMPORT CPI_Metadata	 cpiCreateMetadata( void );
CPIDSOIMPORT CPI_Metadata	 cpiCloneMetadata( const CPI_Metadata meta );
CPIDSOIMPORT CPI_Bool		 cpiMergeMetadata( CPI_Metadata			 result,
											   const CPI_Metadata	 meta );
CPIDSOIMPORT void			 cpiClearMetadata( CPI_Metadata meta );
CPIDSOIMPORT void			 cpiDeleteMetadata( CPI_Metadata meta );

/* removes a particular metadata item from a list */
CPIDSOIMPORT CPI_Bool		 cpiUnsetMetadata( CPI_Metadata	 meta,
											   const char	*name );

/* prints all of the contents of the metadata */
CPIDSOIMPORT void			 cpiPrintMetadata( CPI_Metadata	meta );

CPIDSOIMPORT CPI_Bool		 cpiSetMetaString( CPI_Metadata	 meta,
											   const char	*name,
											   const char	*value );
CPIDSOIMPORT CPI_Bool		 cpiSetMetaBool( CPI_Metadata	 meta,
											 const char		*name,
											 CPI_Bool		 value );
CPIDSOIMPORT CPI_Bool		 cpiSetMetaInt32( CPI_Metadata	 meta,
											  const char	*name,
											  CPI_Int32		 value );
CPIDSOIMPORT CPI_Bool		 cpiSetMetaFloat32( CPI_Metadata meta,
												const char	*name,
												CPI_Float32	 value );
CPIDSOIMPORT CPI_Bool		 cpiSetMetaInt64( CPI_Metadata	 meta,
											  const char	*name,
											  CPI_Int64		 value );
CPIDSOIMPORT CPI_Bool		 cpiSetMetaFloat64( CPI_Metadata	 meta,
												const char		*name,
												CPI_Float64		 value );
CPIDSOIMPORT CPI_Bool		 cpiGetMetaString( const CPI_Metadata  meta,
											   const char			*name,
											   char					*result,
											   size_t				 buffersize );
CPIDSOIMPORT CPI_Bool		 cpiGetMetaBool( CPI_Metadata	 meta,
											 const char		*name,
											 CPI_Bool		*result );
CPIDSOIMPORT CPI_Bool		 cpiGetMetaInt32( const CPI_Metadata	 meta,
											  const char			*name,
											  int					*result );
CPIDSOIMPORT CPI_Bool		 cpiGetMetaFloat32( const CPI_Metadata	 meta,
												const char			*name,
												float				*result );
CPIDSOIMPORT CPI_Bool		 cpiGetMetaInt64( const CPI_Metadata	 meta,
											  const char			*name,
											  int					*result );
CPIDSOIMPORT CPI_Bool		 cpiGetMetaFloat64( const CPI_Metadata	 meta,
												const char			*name,
												float				*result );



/*
/////////////////////////////////////////////////////
// Specialized calls for useful pieces of metadata //
/////////////////////////////////////////////////////
*/


CPIDSOIMPORT CPI_Bool		 cpiSetFilename( CPI_Metadata	 meta,
											 const char		*filename );
CPIDSOIMPORT CPI_Bool		 cpiGetFilename( const CPI_Metadata	 meta,
											 char				*result,
											 size_t				 buffersize );
CPIDSOIMPORT CPI_Bool		 cpiSetDeviceName( CPI_Metadata	 meta,
											   const char	*devname );
CPIDSOIMPORT CPI_Bool		 cpiGetDeviceName( const CPI_Metadata	 meta,
											   char					*result,
											   size_t				 buffersize );
CPIDSOIMPORT CPI_Bool		 cpiSetRead( CPI_Metadata	 meta,
										 CPI_Bool		 onoff );
CPIDSOIMPORT CPI_Bool		 cpiGetRead( const CPI_Metadata	 meta,
										 CPI_Bool			*result );
CPIDSOIMPORT CPI_Bool		 cpiSetWrite( CPI_Metadata	 meta,
										  CPI_Bool		 onoff );
CPIDSOIMPORT CPI_Bool		 cpiGetWrite( const CPI_Metadata	 meta,
										  CPI_Bool				*result );
CPIDSOIMPORT CPI_Bool		 cpiSetTrunc( CPI_Metadata	 meta,
										  CPI_Bool		 onoff );
CPIDSOIMPORT CPI_Bool		 cpiGetTrunc( const CPI_Metadata	 meta,
										  CPI_Bool				*result );
CPIDSOIMPORT CPI_Bool		 cpiSetFileSize( CPI_Metadata	 meta,
											 CPI_Uint64		 filesize );
CPIDSOIMPORT CPI_Bool		 cpiGetFileSize( const CPI_Metadata	 meta,
											 CPI_Uint64			*result );
CPIDSOIMPORT CPI_Bool		 cpiSetFileLastMod( CPI_Metadata	 meta,
												const char		*time );
CPIDSOIMPORT CPI_Bool		 cpiGetFileLastMod( const CPI_Metadata	 meta,
												char				*result,
												size_t				 buffersize );
CPIDSOIMPORT CPI_Bool		 cpiSetIsDir( CPI_Metadata	meta,
										  CPI_Bool		onoff );
CPIDSOIMPORT CPI_Bool		 cpiGetIsDir( const CPI_Metadata	 meta,
										  CPI_Bool				*result );
CPIDSOIMPORT CPI_Bool		 cpiSetIsFile( CPI_Metadata	 meta,
										   CPI_Bool		 onoff );
CPIDSOIMPORT CPI_Bool		 cpiGetIsFile( const CPI_Metadata	 meta,
										   CPI_Bool				*result );



/*
/////////////////////////////////////////////////////
// Specialized calls for Image metadata            //
/////////////////////////////////////////////////////
*/


/*
 * For Global space, if your format supports active area where the
 * file represents a sub-image of a larger space
 */
CPIDSOIMPORT CPI_Bool		cpiSetImageOffset( CPI_Metadata	 meta,
											   CPI_Int32	 xoff,
											   CPI_Int32	 yoff );
CPIDSOIMPORT CPI_Bool		cpiGetImageOffset( CPI_Metadata	 meta,
											   CPI_Int32	*xoff,
											   CPI_Int32	*yoff );

/*
 * For Global space, if your format supports active area where the
 * file represents a sub-image of a larger space
 */
CPIDSOIMPORT CPI_Bool		cpiSetImageSize( CPI_Metadata	 meta,
											 CPI_Int32		 xsize,
											 CPI_Int32		 ysize );
CPIDSOIMPORT CPI_Bool		cpiGetImageSize( CPI_Metadata	 meta,
											 CPI_Int32		*xsize,
											 CPI_Int32		*ysize );

/*
 * The normal case for specifying size....
 */
CPIDSOIMPORT CPI_Bool		cpiSetImageFullSize( CPI_Metadata	 meta,
												 CPI_Int32		 xsize,
												 CPI_Int32		 ysize );
CPIDSOIMPORT CPI_Bool		cpiGetImageFullSize( CPI_Metadata	 meta,
												 CPI_Int32		*xsize,
												 CPI_Int32		*ysize );

/*
 * Other bits of information
 */
CPIDSOIMPORT CPI_Bool		cpiSetImageChannels( CPI_Metadata	 meta,
												 CPI_Int32		 channels );
CPIDSOIMPORT CPI_Bool		cpiGetImageChannels( const CPI_Metadata	 meta,
												 CPI_Int32			*result );

CPIDSOIMPORT CPI_Bool		cpiSetImageBitDepth( CPI_Metadata	 meta,
												 CPI_Int32		 bitdepth );
CPIDSOIMPORT CPI_Bool		cpiGetImageBitDepth( const CPI_Metadata	 meta,
												 CPI_Int32			*result );

CPIDSOIMPORT CPI_Bool		cpiSetImageIsLog( CPI_Metadata	 meta,
											  CPI_Bool		 islog );
CPIDSOIMPORT CPI_Bool		cpiGetImageIsLog( const CPI_Metadata	 meta );

CPIDSOIMPORT CPI_Bool		cpiSetImageIsVideo( CPI_Metadata	 meta,
												CPI_Bool		 isvideo );
CPIDSOIMPORT CPI_Bool		cpiGetImageIsVideo( const CPI_Metadata	 meta );
	
CPIDSOIMPORT CPI_Bool		cpiSetImagePixelRatio( CPI_Metadata	 meta,
												   CPI_Float32	 pixelratio );
CPIDSOIMPORT CPI_Bool		cpiGetImagePixelRatio( const CPI_Metadata   meta,
												   CPI_Float32		   *result );
CPIDSOIMPORT CPI_Bool		cpiSetImageIsPremultiplied( CPI_Metadata	meta,
														CPI_Bool		isPremult );
CPIDSOIMPORT CPI_Bool		cpiGetImageIsPremultiplied( const CPI_Metadata	meta );

/*
 * For formats that support more than one frame in a file
 * or for a collection of files from the file manager
 */
CPIDSOIMPORT CPI_Bool		cpiSetImageRangeStart( CPI_Metadata	 meta,
												   CPI_Int32	 framestart );
CPIDSOIMPORT CPI_Bool		cpiGetImageRangeStart( const CPI_Metadata	 meta,
												   CPI_Int32			*result );

CPIDSOIMPORT CPI_Bool		cpiSetImageRangeEnd( CPI_Metadata	 meta,
												 CPI_Int32		 frameend );
CPIDSOIMPORT CPI_Bool		cpiGetImageRangeEnd( const CPI_Metadata	 meta,
												 CPI_Int32			*result );

CPIDSOIMPORT CPI_Bool		cpiSetImageRangeIncr( CPI_Metadata	 meta,
												  CPI_Int32		 frameincr );
CPIDSOIMPORT CPI_Bool		cpiGetImageRangeIncr( const CPI_Metadata	 meta,
												  CPI_Int32				*result );

CPIDSOIMPORT CPI_Bool		cpiSetImageSubSupport( CPI_Metadata	 meta,
												   CPI_Bool		 onoff );
CPIDSOIMPORT CPI_Bool		cpiGetImageSubSupport( CPI_Metadata	 meta );


/*
 * Formats that can have animated parameters need to be able to get the time
 * that they should evaluate their parameters at
 * NOTE : This time is not set for other plugin types!!!
 */
CPIDSOIMPORT CPI_Bool		cpiGetParmEvalTime( CPI_Metadata	 meta,
												CPI_Float32		*result );


#ifdef __cplusplus
}
#endif

#endif /* _CPI_Metadata_h_ */

