/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	CPI_NodeProvider
//
// COMMENTS:
//
*/

#ifndef _CPI_NodeProvider_h_
#define _CPI_NodeProvider_h_

#include <CPI/CPI_Base.h>

/*////////////////////////////////////// */


#ifdef __cplusplus
extern "C" {
#endif


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the node was initialized successfully
*/
typedef CPI_Bool	 (*RPI_NodeInitFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
//
// Return Value:
// CPI_Bool - should return CPI_TRUE if the node was shut down successfully
*/
typedef CPI_Bool	 (*UPI_NodeShutDownFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	const char *parmname - the name of the parm whose value has been changed.
//						   This gets called with a NULL parmname every time
//						   the node is added.
//
// Return Value:
//	None
*/
typedef void		 (*UPI_NodeParmChangedFn)( const char *parmname );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Uint32 inputnum - The input number ( 0 based ) which has been changed,
//						  (replaced, deleted)
//
// Return Value:
//	Node
*/
typedef void		 (*UPI_NodeInputChangedFn)( CPI_Uint32 inputnum );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_ImageContext *region - This structure needs to be filled out with the
//							   region your node will output
//	CPI_Float32 myTime - The time at which rayz is requesting the output
//						 region
//	CPI_Uint32 output - Rayz has the ability for a node to contain multiple
//						different outputs, this is the numerical value ( 0
//						based ) of the output were getting the region for.
//	CPI_Bool viewerOutput - This will tell you whether the output number given
//							is for a viewer output or for your nodes output.
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the region was successfully obtained
//
// Undefined Behavior:
//	If this function is not provided then the output region will be assumed to
//	be the same as the input region.
*/
typedef CPI_Bool	 (*UPI_NodeGetRegionFn)( CPI_ImageContext	*region,
											 CPI_Float32		 myTime,
											 CPI_Uint32		 	 output,
											 CPI_Bool			 viewerOutput );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_ImageContext *fullsize - This structure only needs to have the sizeX
//								 and sizeY members filled out
//	CPI_Float32 myTime - The time at which rayz is requesting the full size of
//						 your node
//  CPI_Uint32 output - Rayz has the ability for a node to contain multiple
//						different outputs, this is the numerical value ( 0
//						based ) of the output we're getting the full size
//					    for.
//	CPI_Bool viewerOutput - This will tell you whether the output number given
//							is for a viewer output or for your nodes output.
//                          This only matters if you have added extra viewer
//							outputs to your node
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the full size was successfully
//			   obtained
//
// Undefined Behavior:
//	If this function is not provided the output full size will be assumed to
//	be the same as the input full size.
*/
typedef CPI_Bool	 (*UPI_NodeGetFullSizeFn)( CPI_ImageContext	*fullsize,
											   CPI_Float32		 myTime,
											   CPI_Uint32		 output,
											   CPI_Bool			 viewerOutput );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Int32 outStart - Starting Frame Number of your current nodes sequence
//	CPI_Int32 outEnd - Ending Frame Number of your current nodes sequence
//	CPI_Uint32 output - Rayz has the ability for a node to contain multiple
//						different outputs, this is the numerical value ( 0
//						based ) of the output were getting the range for.
//	CPI_Bool viewerOutput - This will tell you whether the output number given
//							is for a viewer output or for your nodes output.
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the sequence was successfully obtained
//
// Undefined Behavior:
//	If this function is not provided the output frame range will be assumed to
//	be the same as the input frame range.
*/
typedef	CPI_Bool	 (*UPI_NodeGetRangeFn)( CPI_Int32	*outStart,
											CPI_Int32	*outEnd,
											CPI_Uint32	 output,
											CPI_Bool	 viewerOutput );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Float32 time - This is the time that your should check for errors at
//
// Return Value:
//	CPI_Bool - return CPI_FALSE if you encountered an error
//
// Undefined Behavior:
//	If this function is not provided then the cook will get called as normal.
//	This function is a shortcut provided in order to save processing time when
//	there is an obvious error.
*/
typedef CPI_Bool	(*UPI_NodeGetErrorsFn)( CPI_Float32 time );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Float32 time - This the the time rayz is asking you to execute at
//
//	CPI_Uint8 quality - This is the quality at which rayz has asked you to
//						execute.  It will be CPI_HIGH_QUALITY,
//						CPI_MEDIUM_QUALITY or CPI_LOW_QUALITY.
//
//	CPI_Uint32 output - This is the output number which rayz is requesting
//						you execute.  If viewerOutput is CPI_TRUE then this
//						is the viewerOutput number, if it's CPI_FALSE then
//						this is the node output number.
//	CPI_Bool viewerOutput - This will tell you whether the output number given
//							is for a viewer output or for your node's output.
//	CPI_Float32 scaleX - X scale value at which rayz has asked you to cook.
//	CPI_Float32 scaleY - Y scale value at which rayz has asked you to cook.
// Return Value:
//	CPI_ImageOp - The return of the last call to cpiAddImageOp() or
//				  cpiGetInputOp() should be returned.  If NULL is returned,
//				  the node will not cook.
*/
typedef CPI_ImageOp	 (*RPI_NodeExecutionFn)( CPI_Float32	time,
											 CPI_Uint8		quality,
											 CPI_Uint32		output,
											 CPI_Bool		viewerOutput,
											 CPI_Float32	scaleX,
											 CPI_Float32 	scaleY );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Uint32 input - This is the input number ( 0 based ) which you are
//					   setting the label for
//
// Return Value:
//	const char * - This is the label you want to be placed on your input
//
// Undefined Behavior -
//	Your input will appropriately be labeled 'input'
*/
typedef const char	*(*UPI_NodeInputLabelFn)( CPI_Uint32	input );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Uint32 output - This is the output number ( 0 based ) which you are
//						setting the label for
//
// Return Value:
//	const char * - This is the label you want to be placed on your output
//
// Undefined Behavior -
//	Your output will appropriately be labeled  'output'
*/
typedef const char	*(*UPI_NodeOutputLabelFn)( CPI_Uint32	output );

/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	CPI_Uint32 whichViewer - Which of your viewer outputs you are setting the
//							 label for (NOTE : You are only able to change the
//							 viewer label for outputs which your have added)
// Return Value:
//	const char * - This is the label you want to be placed on your viewer
//				   selection for which output to view
//
// Undefined Behavior -
//	Your output will appropriately be labeled  'output'
*/
typedef const char	*(*UPI_NodeViewerLabelFn)( CPI_Uint32	whichViewer );

/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
// Return Value:
//	CPI_Uint32 - This is the number of extra viewer outputs you want to have
//
// Undefined Behavior -
//	Your node will not have any extra viewer outputs
*/
typedef CPI_Uint32	(*UPI_NodeNumViewerOutputsFn)( void );


/*
// For any optional (upi labeled) functions you don't wish to provide,
// fill in NULL
*/
typedef struct
{
	RPI_PluginInfo				rpiPluginInfo;

	// Functions for initialization and other changes
	RPI_NodeInitFn				rpiNodeInit;
	UPI_NodeShutDownFn			upiNodeShutDown;
	UPI_NodeParmChangedFn		upiNodeParmChanged;
	UPI_NodeInputChangedFn		upiNodeInputChanged;

	// Functions for getting the current status and setting the exec list
	UPI_NodeGetRegionFn			upiNodeGetRegion;
	UPI_NodeGetFullSizeFn		upiNodeGetFullSize;
	UPI_NodeGetRangeFn			upiNodeGetRange;
	UPI_NodeGetErrorsFn			upiNodeGetErrors;
	RPI_NodeExecutionFn			rpiNodeExecute;

	// Functions for setting UI attributes about the node
	UPI_NodeInputLabelFn		upiNodeInputLabel;
	UPI_NodeOutputLabelFn		upiNodeOutputLabel;
	UPI_NodeViewerLabelFn		upiNodeViewerLabel;
	UPI_NodeNumViewerOutputsFn	upiNodeViewerOutputs;

	// Node characteristics
	CPI_Uint32					upiMinInputs;
	CPI_Uint32					upiMaxInputs;
	CPI_Uint32					upiMinOutputs;
	CPI_Uint32					upiMaxOutputs;
	CPI_Bool					upiIsGrowable;
} RPI_NodeInfo;

#ifdef __cplusplus
}
#endif

#endif /* _CPI_NodeProvider_h_ */
