/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
*/

#ifndef _CPI_Overlay_h_
#define _CPI_Overlay_h_

#include <CPI/CPI_Types.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Alignment constants for drawing Text */
#define CPI_ALIGN_LEFT				0
#define CPI_ALIGN_CENTER			1
#define CPI_ALIGN_RIGHT				2
#define CPI_ALIGN_BOTTOM_LEFT		3
#define CPI_ALIGN_BOTTOM_CENTER		4
#define CPI_ALIGN_BOTTOM_RIGHT		5
#define CPI_ALIGN_TOP_LEFT			6
#define CPI_ALIGN_TOP_CENTER		7
#define CPI_ALIGN_TOP_RIGHT			8

/*
// These are the drawing routines available to the Rayz programmer
// All x,y values, radii, and dimensions are given in a dimensionless
// 0.0 to 1.0 space.
//
// Section 1 - Geometry 
*/

CPIDSOIMPORT void			 cpiDrawLine( const CPI_Float32 x1,
										  const CPI_Float32 y1,
										  const CPI_Float32 x2,
										  const CPI_Float32 y2 );
CPIDSOIMPORT void			 cpiDrawRect( const CPI_Float32 x,
										  const CPI_Float32 y,
										  const CPI_Float32 w,
										  const CPI_Float32 h );
CPIDSOIMPORT void			 cpiDrawFilledRect( const CPI_Float32 x,
											    const CPI_Float32 y,
											    const CPI_Float32 w,
											    const CPI_Float32 h );
CPIDSOIMPORT void			 cpiDrawCircle( const CPI_Float32 x,
										    const CPI_Float32 y,
										    const CPI_Float32 r );
CPIDSOIMPORT void			 cpiDrawFilledCircle( const CPI_Float32 x,
												  const CPI_Float32 y,
												  const CPI_Float32 r );
CPIDSOIMPORT void			 cpiDrawEllipse( const CPI_Float32 x,
											 const CPI_Float32 y,
											 const CPI_Float32 rx,
											 const CPI_Float32 ry );
CPIDSOIMPORT void			 cpiDrawFilledEllipse( const CPI_Float32 x,
												   const CPI_Float32 y,
												   const CPI_Float32 rx,
												   const CPI_Float32 ry );
CPIDSOIMPORT void			 cpiDrawArc( const CPI_Float32 x,
										 const CPI_Float32 y,
										 const CPI_Float32 r,
										 const CPI_Float32 a,
										 const CPI_Float32 b );
CPIDSOIMPORT void			 cpiDrawFilledArc( const CPI_Float32 x,
											   const CPI_Float32 y,
											   const CPI_Float32 r,
											   const CPI_Float32 a,
											   const CPI_Float32 b );

/* Define an arbitrary line */
CPIDSOIMPORT void			 cpiBeginLine( void );
CPIDSOIMPORT void			 cpiLineVertex( const CPI_Float32 x,
										    const CPI_Float32 y );
CPIDSOIMPORT void			 cpiEndLine( void );

/* Define an arbitrary polygon, as outline or solid filled shape */
CPIDSOIMPORT void			 cpiBeginPoly( void );
CPIDSOIMPORT void			 cpiBeginFilledPoly( void );
CPIDSOIMPORT void			 cpiPolyVertex( const CPI_Float32 x,
										    const CPI_Float32 y );
CPIDSOIMPORT void			 cpiEndPoly( void );

/* Define an arbitrary bezier shape */
CPIDSOIMPORT void			 cpiBeginBezier( void );
CPIDSOIMPORT void			 cpiBezierVertex( const CPI_Float32 x,
											  const CPI_Float32 y );
CPIDSOIMPORT void			 cpiEndBezier( void );

/*
// Section 2 - Text routines
// alignment can be one of:
//			CPI_ALIGN_LEFT
//			CPI_ALIGN_CENTER
//			CPI_ALIGN_RIGHT
//			CPI_ALIGN_BOTTOM_LEFT
//			CPI_ALIGN_BOTTOM_CENTER
//			CPI_ALIGN_BOTTOM_RIGHT
//			CPI_ALIGN_TOP_LEFT
//			CPI_ALIGN_TOP_CENTER
//			CPI_ALIGN_TOP_RIGHT
*/
CPIDSOIMPORT void			 cpiDrawText( const CPI_Uint32	 alignment,
										  const CPI_Float32	 x,
										  const CPI_Float32	 y,
										  const CPI_Float32	 w,
										  const CPI_Float32	 h,
										  const char			*msg );

/*
// Section 3 - Utility functions
// Start the definition of a new object. All subsequent graphics calls
// will be associated with this object ID. It is not necessary to close
// an object - starting an object implies closing the previous one.
*/
CPIDSOIMPORT void			 cpiStartNewObject( CPI_Uint32 objectID );

/*
// Set the current color. This color is associated with all subsequent
// graphics calls
*/
CPIDSOIMPORT void			 cpiSetColor( const CPI_Float32	 red,
										  const CPI_Float32	 green,
										  const CPI_Float32	 blue );
/*
// Force a redraw of all defined objects - programmers must explicitly
// call this routine
*/
CPIDSOIMPORT void			 cpiDoRedraw( void );

/* Lock the current object, to prevent picking */
CPIDSOIMPORT void			 cpiSetSelectLock( CPI_Bool lock );

#ifdef __cplusplus
}
#endif

#endif /* _CPI_Overlay_h_ */
