/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	CPI_NodeProvider
//
// COMMENTS:
//
*/


#ifndef _CPI_OverlayProvider_h_
#define _CPI_OverlayProvider_h_

#include <CPI/CPI_Base.h>

/*
// These flags will be passed as keyMod into each of the mouse handling
// functions
*/
#define CPI_CTRL_KEY	0x0001
#define CPI_ALT_KEY		0x0002
#define CPI_SHIFT_KEY	0x0004


/*////////////////////////////////////// */


#ifdef __cplusplus
extern "C" {
#endif

typedef CPI_Bool		(*UPI_OverlayInit)( void );
typedef CPI_Bool		(*UPI_OverlayShutdown)( void );
typedef CPI_PrivateData (*UPI_OverlayCreateInstance)( void );
typedef void			(*UPI_OverlayDestroyInstance)( CPI_PrivateData handle );
typedef void			(*UPI_OverlaySourceChanged)( CPI_PrivateData  handle,
													 CPI_ImageContext fullsize,
													 CPI_Float32	  time );
typedef void			(*UPI_OverlayFullSizeChanged)( CPI_PrivateData	handle,
													   CPI_ImageContext	fullsize,
													   CPI_Float32		time );
typedef void			(*UPI_OverlaySourceOutOfDate)( CPI_PrivateData	handle ,
													   CPI_ImageContext	fullsize,
													   CPI_Float32		time );
typedef void			(*RPI_OverlayDoRender)( CPI_PrivateData	handle,
												CPI_Bool		isPicking );
typedef void			(*UPI_OverlayLeftMouseDown)( CPI_PrivateData handle,
													 CPI_Uint32	objectID,
													 CPI_Float32	x,
													 CPI_Float32	y,
													 CPI_Uint32	keyMod );
typedef void			(*UPI_OverlayLeftMouseDrag)( CPI_PrivateData handle,
													 CPI_Uint32	objectID,
													 CPI_Float32	x,
													 CPI_Float32	y,
													 CPI_Uint32	keyMod,
													 CPI_Bool	isContinuous );
typedef void			(*UPI_OverlayLeftMouseUp)( CPI_PrivateData	handle,
												   CPI_Uint32		objectID,
												   CPI_Float32		x,
												   CPI_Float32		y,
												   CPI_Uint32		keyMod );
typedef void			(*UPI_OverlayMouseEnter)( CPI_PrivateData	handle,
												  CPI_Uint32		objectID,
												  CPI_Float32		x,
												  CPI_Float32		y );
typedef void			(*UPI_OverlayMouseDrag)( CPI_PrivateData	handle,
												 CPI_Float32		x,
												 CPI_Float32		y );
typedef void			(*UPI_OverlayMouseExit)( CPI_PrivateData	handle,
												 CPI_Float32		x,
												 CPI_Float32		y );

typedef struct
{
	RPI_PluginInfo			 rpiPluginInfo;

	// Functions for initialization and change events
	UPI_OverlayInit					 upiOverlayInit;
	UPI_OverlayShutdown				 upiOverlayShutDown;
	UPI_OverlayCreateInstance		 upiOverlayCreateInstance;
	UPI_OverlayDestroyInstance		 upiOverlayDestroyInstance;
	UPI_OverlaySourceChanged		 upiOverlaySourceChanged;
	UPI_OverlayFullSizeChanged		 upiOverlayFullSizeChanged;
	UPI_OverlaySourceOutOfDate		 upiOverlaySourceOutOfDate;
	RPI_OverlayDoRender				 rpiOverlayDoRender;
	UPI_OverlayLeftMouseDown		 upiOverlayLeftMouseDown;
	UPI_OverlayLeftMouseDrag		 upiOverlayLeftMouseDrag;
	UPI_OverlayLeftMouseUp			 upiOverlayLeftMouseUp;
	UPI_OverlayMouseEnter			 upiOverlayMouseEnter;
	UPI_OverlayMouseDrag			 upiOverlayMouseDrag;
	UPI_OverlayMouseExit			 upiOverlayMouseExit;

	// If this overlay is for general purpose UI, then pass in NULL
	// for the node name
	const char						*rpiOverlayNodeName;

	// If this is CPI_TRUE, you will receive mouse events even when the mouse
	// is outside any area rendered during the pick render and you are the top
	// fullscreen overlay
	CPI_Bool						 rpiFullscreenOverlay;

} RPI_OverlayInfo;

#ifdef __cplusplus
}
#endif

#endif /* _CPI_OverlayProvider_h_ */
