/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
*/

#ifndef _CPI_Parameters_h_
#define _CPI_Parameters_h_

#include <CPI/CPI_Types.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Various flags for the parameters */
#define CPI_PARM_DEFAULT			0x00000000
#define CPI_PARM_ANIMATABLE			0x00000001
#define CPI_PARM_INVISIBLE			0x00000002
#define CPI_PARM_NOMODIFIES			0x00000004
#define CPI_PARM_EXPANDBYDEFAULT	0x00000008
#define CPI_PARM_CLAMPMIN			0x00000010
#define CPI_PARM_CLAMPMAX			0x00000020
#define CPI_PARM_CLAMPMINMAX		(CPI_PARM_CLAMPMIN | CPI_PARM_CLAMPMAX)
#define CPI_PARM_READONLY			0x00000040
#define CPI_PARM_FULL_SIZE_X		0x00000080	/* NOTE: Float parms only! */
#define CPI_PARM_FULL_SIZE_Y		0x00000100	/* NOTE: Float parms only! */
#define CPI_PARM_FULL_SIZE_XY		0x00000200	/* NOTE: Float pairs only! */
#define CPI_PARM_PIXEL_VALUE		0x00000400	/* NOTE: Float parms only! */
	
/* Flags used for channel select parameters */
#define CPI_PARM_SELECT_NONE		0x00
#define CPI_PARM_SELECT_RED			0x01
#define CPI_PARM_SELECT_GREEN		0x02
#define CPI_PARM_SELECT_BLUE		0x04
#define CPI_PARM_SELECT_ALPHA		0x08
#define CPI_PARM_SELECT_OTHER		0x10

CPIDSOIMPORT void			 cpiAddMenu( const char		*name,
										 const char		*label,
										 CPI_Uint8		 def,
										 const char		*entries[],
										 const char		*helpText,
										 const char		*helpHTML );
CPIDSOIMPORT void			 cpiAddInteger( const char		*name,
											const char		*label,
											CPI_Int32		 def,
											CPI_Int32		 rngmin,
											CPI_Int32		 rngmax,
											CPI_Int32		 flags,
											const char		*helpText,
											const char		*helpHTML );
CPIDSOIMPORT void		 	 cpiAddFloat( const char	*name,
										  const char	*label,
										  CPI_Float32	 def,
										  CPI_Float32	 rngmin,
										  CPI_Float32	 rngmax,
										  CPI_Int32		 flags,
										  const char	*helpText,
										  const char	*helpHTML );
CPIDSOIMPORT void			 cpiAddToggle( const char	*name,
										   const char	*label,
										   CPI_Bool		 def,
										   CPI_Int32	 flags,
										   const char	*helpText,
										   const char	*helpHTML );
CPIDSOIMPORT void			 cpiStartGroup( const char	*name,
											const char	*label,
											CPI_Int32	 flags,
											const char	*helpText,
											const char	*helpHTML );
CPIDSOIMPORT void			 cpiEndGroup( void );
CPIDSOIMPORT void			 cpiAddFile( const char		*name,
										 const char		*label,
										 const char		*def,
										 const char		*helpText,
										 const char		*helpHTML );
CPIDSOIMPORT void			 cpiAddColorGroup( const char	*name,
											   const char	*label,
											   CPI_Float32	 def[],
											   CPI_Uint32	 numChans,
											   CPI_Int32	 flags,
											   const char	*helpText,
											   const char	*helpHTML );
CPIDSOIMPORT void			 cpiAddFloatRGBAOGroup( const char	*name,
													const char	*label,
													CPI_Float32	 def,
													CPI_Float32	 rngmin,
													CPI_Float32	 rngmax,
													CPI_Uint32	 entries,
													CPI_Int32	 flags,
													const char	*helpText,
													const char	*helpHTML );
CPIDSOIMPORT void			 cpiAddIntRGBAOGroup( const char	*name,
												  const char	*label,
												  CPI_Int32		 def,
												  CPI_Int32		 rngmin,
												  CPI_Int32		 rngmax,
												  CPI_Uint32	 numentries,
												  CPI_Int32		 flags,
												  const char	*helpText,
												  const char	*helpHTML );
CPIDSOIMPORT void			 cpiAddIntPair( const char	*nameTop,
											const char	*labelTop,
											const char	*nameOne,
											const char	*labelOne,
											CPI_Int32	 defaultOne,
											const char	*nameTwo,
											const char	*labelTwo,
											CPI_Int32	 defaultTwo,
											CPI_Int32	 flags,
											const char	*helpText,
											const char	*helpHTML );
CPIDSOIMPORT void			 cpiAddFloatPair( const char	*nameTop,
											  const char	*labelTop,
											  const char	*nameOne,
											  const char	*labelOne,
											  CPI_Float32	 defaultOne,
											  const char	*nameTwo,
											  const char	*labelTwo,
											  CPI_Float32	 defaultTwo,
											  CPI_Int32		 flags,
											  const char	*helpText,
											  const char	*helpHTML );
CPIDSOIMPORT void			 cpiAddHueCircle( const char	*name,
											  const char	*label,
											  CPI_Float32	 def,
											  CPI_Int32		 flags,
											  const char	*helpText,
											  const char	*helpHTML );
CPIDSOIMPORT void			 cpiStartVarList( void );
CPIDSOIMPORT void			 cpiDefineVarListGroup( const char	*name,
													const char	*label,
													void		*addFunction );
CPIDSOIMPORT void			 cpiEndVarList( const char	*name,
											const char	*label,
											CPI_Int32	 flags,
											CPI_Bool	 addButtons );
CPIDSOIMPORT void			 cpiAddChannelSelect( const char	*helpText,
												  const char	*helpHTML );
CPIDSOIMPORT void			 cpiAddString( const char	*name,
										   const char	*label,
										   const char	*def,
										   CPI_Bool		 multiLine,
										   CPI_Int32	 flags,
										   const char	*helpText,
										   const char	*helpHTML );
CPIDSOIMPORT void			 cpiAddDisplayString( const char	*name,
												  const char	*label,
												  const char	*defString );
CPIDSOIMPORT void			 cpiAddButton( const char			*name,
										   const char			*label,
										   CPI_CallBackFn		 buttonFunc,
										   CPI_Int32			 flags,
										   const char			*helpText,
										   const char			*helpHTML );
CPIDSOIMPORT void			 cpiAddSequenceRange( CPI_Int32		 startFrameDef,
												  CPI_Int32		 endFrameDef,
												  CPI_Int32		 flags,
												  const char	*helpText,
												  const char	*helpHTML );
CPIDSOIMPORT CPI_Float32	 cpiGetCurrentXScale( void );
CPIDSOIMPORT CPI_Float32	 cpiGetCurrentYScale( void );

CPIDSOIMPORT CPI_Bool		 cpiGetFloat( CPI_Float32		*fltval,
										  const char		*parmname,
										  CPI_Float32		 evaltime );

CPIDSOIMPORT CPI_Bool		 cpiGetInteger( CPI_Int32		*intval,
											const char		*parmname,
											CPI_Float32		 evaltime );

CPIDSOIMPORT CPI_Bool		 cpiGetString( char			*stringval,
										   CPI_Int32	 stringsize,
										   const char	*parmname,
										   CPI_Float32	 evaltime );

CPIDSOIMPORT CPI_Bool		 cpiGetChannelSelect( CPI_Int32	*channelval );

CPIDSOIMPORT CPI_Bool		 cpiGetSequenceParmRange( CPI_Int32	*startFrame,
													  CPI_Int32	*endFrame );
	
CPIDSOIMPORT CPI_Bool		 cpiSetFloat( const CPI_Float32	 fltval,
										  const char	 	*parmname,
										  CPI_Float32		 evaltime,
										  CPI_Bool			 isUserSet );
CPIDSOIMPORT CPI_Bool		 cpiSetInteger( const CPI_Int32	 intval,
											const char		*parmname,
											CPI_Float32		 evaltime,
											CPI_Bool		 isUserSet );
CPIDSOIMPORT CPI_Bool		 cpiSetString( const char		*stringval,
										   const char		*parmname,
										   CPI_Float32		 evaltime,
										   CPI_Bool			 isUserSet );
CPIDSOIMPORT CPI_Bool		 cpiSetSequenceParmRange( CPI_Int32	 startFrame,
													  CPI_Int32	 endFrame );
	
CPIDSOIMPORT CPI_Bool		 cpiIsParmUserSet( const char *parmname );
CPIDSOIMPORT void		 	 cpiEnableParameter( const char	*parmname,
												 CPI_Bool	 enable );


CPIDSOIMPORT CPI_Bool		 cpiGetInputContext( CPI_ImageContext	*context,
												 CPI_Uint32			whichInput,
												 CPI_Float32		 theTime );
CPIDSOIMPORT CPI_Bool	  	 cpiGetInputFullSize( CPI_Int32		*x,
												  CPI_Int32	 	*y,
												  CPI_Uint32	 whichInput,
												  CPI_Float32	 theTime );
CPIDSOIMPORT CPI_Bool		 cpiGetInputRange( CPI_Int32	*start,
											   CPI_Int32	*end,
											   CPI_Uint32	 whichInput );
CPIDSOIMPORT CPI_Bool	 	 cpiGetNumInputs( CPI_Uint32	*numinputs );

CPIDSOIMPORT CPI_Bool		 cpiAddInput( void );
	
CPIDSOIMPORT CPI_Bool		 cpiRemoveInput( CPI_Uint32	whichInput );
	
CPIDSOIMPORT CPI_Bool	 	 cpiAddVarListGroup( const char	*varlistname,
												 const char	*entryname );

CPIDSOIMPORT CPI_Bool		 cpiRemoveVarListGroup( const char *varlistname,
													CPI_Uint32	entrynum );

CPIDSOIMPORT CPI_Bool		 cpiMoveVarListGroup( const char	*varlistname,
												  CPI_Uint32	 entryToMove,
												  CPI_Uint32	 moveBefore );
	
CPIDSOIMPORT CPI_Bool	  	 cpiGetVarListSize( const char	*varlistname,
												CPI_Uint32	*size );	

CPIDSOIMPORT CPI_Bool		 cpiIsInputConnected( CPI_Uint32 input );

CPIDSOIMPORT CPI_Bool		 cpiGetGlobalFrameRange( CPI_Int32	*startFrame,
													 CPI_Int32	*endFrame );
	
#ifdef __cplusplus
}
#endif

#endif /* _CPI_Parameters_h_ */

