/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// Comments: Functions useable to interface with the preference editor
//  and associated user-editable preference values
*/

#ifndef _CPI_Preferences_h_
#define _CPI_Preferences_h_

#include <CPI/CPI_Types.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * These two functions exist to enable a plugin developer to define
 * preferences that will then be editable in the preferences editor
 * and project settings dialogs in Rayz, given the appropriate path.
 *
 * If a preference is defined to be in the "Project" group, or a sub
 * group of "Project", the preferences will appear in the project settings
 * dialog, and non-default values will be saved in each .rayz project
 * file. If the preference is in "User Interface", or a sub group of,
 * it will appear in the preferences editor, and will be saved on a
 * user by user basis in the ~/.rayz/<rayz version>/Preferences/Defaults.pref
 *
 * A preference path is very similar to a unix file path, although the
 * leading / is not needed. Spaces are allowed.
 * So "User Interface/Settings/Help Tag Enable" is the path to define
 * whether help tags are on or off.
 *
 * Preferences are all kept in string form, but convenience functions are
 * provided for converting to other types, if the preference can be
 * parsed as that type
 */
#define CPI_PREF_FLOAT	0
#define CPI_PREF_INT	1
#define CPI_PREF_BOOL	2
#define CPI_PREF_STRING	3
#define CPI_PREF_FILE	4
#define CPI_PREF_COLOR	5

#define CPI_PREF_UI_DEFAULT			0
/*
 * Indicates that the file browser provided in the pref editor should
 * only allow searching for a directory
 */
#define CPI_PREF_UI_FILE_DIR_ONLY	1
/*
 * Indicates that the file browser provided in the pref editor should
 * only allow searching for a file
 */
#define CPI_PREF_UI_LOCAL_ONLY		2
/*
 * Indicates that the file browser provided in the pref editor should
 * only allow searching for a directory that is on the system filesystem
 */
#define CPI_PREF_UI_LOCAL_DIR_ONLY	3

CPIDSOIMPORT void		cpiAddPreferenceGroup( const char	*prefPath );
CPIDSOIMPORT void		cpiAddPreferenceValue( const char	*prefGroup,
											   const char	*prefName,
											   const char	*prefDescription,
											   const char	*defPrefVal,
											   CPI_Int32	 prefType,
											   CPI_Int32	 prefUIType );

CPIDSOIMPORT CPI_Bool	cpiGetPreferenceString( char		*tmpbuf,
												CPI_Uint32	 bufSize,
												const char	*prefPath );

CPIDSOIMPORT CPI_Bool	cpiGetPreferenceInt( CPI_Int32		*val,
											 const char		*prefPath );

CPIDSOIMPORT CPI_Bool	cpiGetPreferenceFloat( CPI_Float32	*val,
											   const char	*prefPath );

CPIDSOIMPORT CPI_Bool	cpiGetPreferenceBool( CPI_Bool		*val,
											  const char	*prefPath );

CPIDSOIMPORT CPI_Bool	cpiGetPreferenceColor( CPI_Float32	*redVal,
											   CPI_Float32	*greenVal,
											   CPI_Float32	*blueVal,
											   const char	*prefPath );

CPIDSOIMPORT CPI_Bool	cpiGetPreferenceFile( char			*fileId,
											  CPI_Uint32	 fileIdBufSize,
											  char			*fileMgr,
											  CPI_Uint32	 fileMgrBufSize,
											  const char	*prefPath );

CPIDSOIMPORT void		 cpiSetPreferenceString( const char	*prefPath,
												 const char	*prefStr );
CPIDSOIMPORT void		 cpiSetPreferenceInt( const char	*prefPath,
											  CPI_Int32		 val );
CPIDSOIMPORT void		 cpiSetPreferenceFloat( const char	*prefPath,
												CPI_Float32	 val );
CPIDSOIMPORT void		 cpiSetPreferenceBool( const char	*prefPath,
											   CPI_Bool		 val );
CPIDSOIMPORT void		 cpiSetPreferenceColor( const char	*prefPath,
												CPI_Float32	 redVal,
												CPI_Float32	 greenVal,
												CPI_Float32	 blueVal );
CPIDSOIMPORT void		 cpiSetPreferenceFile( const char	*prefPath,
											   const char	*fileID,
											   const char	*fileMgr );

#ifdef __cplusplus
}
#endif

#endif /* _CPI_Preferences_h_ */

