/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
//
// NAME:	CPI_ScriptProvider
//
// COMMENTS:
//
*/


#ifndef _CPI_ScriptProvider_h_
#define _CPI_ScriptProvider_h_

#include <CPI/CPI_Base.h>
#include <CPI/CPI_Device.h>


/*////////////////////////////////////// */


#ifdef __cplusplus
extern "C" {
#endif

/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the script entry was initialized
//			   successfully
*/
typedef CPI_Bool	 (*RPI_ScriptInitFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//	None
//
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the script entry was shut down
//			   successfully
*/
typedef CPI_Bool	 (*UPI_ScriptShutDownFn)( void );


/*
///////////////////////////////////////////////////////////////////////////////
//
// Arguments:
//  CPI_DeviceHandle *device - This is a handle to file which you should
//							   fill with the neccessary data.
//	const char *inputName - This is the name of the input node which would be
//							upstream whatever your script adds.
//							NOTE : If your option is the first in the list,
//							the inputName will be passed in as NULL.
//	CPI_Uint32	numArgs - This is the number of arguments which the user
//						  passed into your script option.  If you require
//						  more arguments then your were passed, you should
//						  add a cpiError and return CPI_FALSE;
//	const char **arguments - These are the arguments which were passed into
//							 your script option.
//	const char **outputName - This is the name of the last node which you
//							  have created and is used to specify connections
//							  (i.e. will be passed in as the inputName to an
//							  another node). If the node being created doesn't
//							  have an output (i.e. imageout), this should be
//							  NULL.
//							  NOTE : You should cpiAlloc this pointer or set it
//							  to the return value of cpiGetUniqueNodeName(). Rayz
//							  will be responsible for freeing it, unless return
//							  value is false.
// Return Value:
//	CPI_Bool - should return CPI_TRUE if the script entry was processed
//			   successfully
*/
typedef CPI_Bool 	(*RPI_ScriptProcessFn)( CPI_DeviceHandle	  device,
											const char			 *inputName,
											CPI_Uint32 			  numArgs,
											const char			**arguments,
											const char			**outputName );

/*
// For any optional (upi labeled) functions you don't wish to provide,
// fill in NULL
*/

typedef struct
{
	RPI_PluginInfo				 rpiPluginInfo;

	RPI_ScriptInitFn			 rpiScriptInit;
	UPI_ScriptShutDownFn		 upiScriptShutDown;

	RPI_ScriptProcessFn			 rpiScriptProcess;

	const char					*rpiScriptUsage;
	CPI_Bool					 rpiRenderByDefault;
} RPI_ScriptInfo;

#ifdef __cplusplus
}
#endif

#endif /* _CPI_ScriptProvider_h_ */
