/*
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
// Produced by:
//  Silicon Grail Corporation
//  736 Seward St
//  Hollywood, CA
//  USA 90038
//  323-871-9100
*/

#ifndef _CPI_Types_h_
#define _CPI_Types_h_

#ifdef __cplusplus
extern "C" {
#endif

/*
// The wonderful Win32 DLL mechanism requires one of
// a couple of mechanisms to denote what is externalized
// to avoid having to update functions in a couple of places
// we just declare them as exported as part of their linkage
// which seems to be the easier way maintenance wise...
//
// So, in order to have this happen in your code, just add a
// CPIDSOEXPORT in front of all 'public' functions - i.e.
// rpiPluginInit and upiPluginCleanup
*/
#ifdef WIN32
# ifndef CPIDSOEXPORT
#  define CPIDSOEXPORT __declspec( dllexport )
# endif
# ifndef CPIDSOIMPORT
//#  define CPIDSOIMPORT extern __declspec( dllimport )
#  define CPIDSOIMPORT extern
# endif
#else
# ifndef CPIDSOEXPORT
#  define CPIDSOEXPORT
# endif
# ifndef CPIDSOIMPORT
#  define CPIDSOIMPORT extern
# endif
#endif


typedef void				*CPI_Metadata;
typedef void				*CPI_PrivateData;
typedef void				*CPI_DeviceHandle;
typedef void				*CPI_FileIDList;
typedef void				*CPI_FileIDSeqList;
typedef void				*CPI_ImageOp;
typedef void				*CPI_CurveIDList;

typedef int					 CPI_Bool;
typedef signed char			 CPI_Int8;
typedef signed short		 CPI_Int16;
typedef signed int			 CPI_Int32;

#ifdef WIN32
typedef signed __int64		 CPI_Int64;
#else
typedef signed long long	 CPI_Int64;
#endif

typedef unsigned char	 	 CPI_Uint8;
typedef unsigned short		 CPI_Uint16;
typedef unsigned int		 CPI_Uint32;
#ifdef WIN32
typedef unsigned __int64	 CPI_Uint64;
#else
typedef unsigned long long	 CPI_Uint64;
#endif

typedef float				 CPI_Float32;
typedef double				 CPI_Float64;


/* Defines */
#define CPI_STRING_LENGTH 2048
#define CPI_PATH_LENGTH 2048

#define CPI_TRUE 1
#define CPI_FALSE 0

#define CPI_HIGH_QUALITY	2
#define CPI_MEDIUM_QUALITY	1
#define CPI_LOW_QUALITY		0

#define CPI_LINEAR_IMAGE	0
#define CPI_LOG_IMAGE		1
#define CPI_VIDEO_IMAGE		2

/* ////////////////////////////////// */


typedef struct
{
	CPI_Int32		 sizeX;
	CPI_Int32		 sizeY;
	CPI_Int32		 offsetX;
	CPI_Int32		 offsetY;
	CPI_Int32		 channels;
	CPI_Int32		 skipPixels;	/* Number of pixels between each scanline
									 * to avoid any issues, if this is non-0
									 * it will be easiest to use cpiGetLine()
									 * to retrieve a pointer to the line you
									 * wish...
									 */
	CPI_Int32		 bitsPerPel;	/* Will be passed in as 8, 16, or 32 */
	CPI_Float32		 pixelRatio;
	CPI_Int32		 dataType;		/* Will be passed in as 0 = linear,
									 *                      1 = log,
									 *						2 = video
									 * use #defines CPI_LINEAR_IMAGE
									 * 				CPI_LOG_IMAGE
									 * 				CPI_VIDEO_IMAGE
									 */
	CPI_Float32		 referenceBlack;
	CPI_Float32		 referenceWhite;
	CPI_Float32		 linearWhite;
	CPI_Float32		 displayGamma;
	CPI_Float32		 filmGamma;
	CPI_Float32		 videoGamma;
	CPI_Bool		 isPremultiplied;
	CPI_Float32		 backingRed;
	CPI_Float32		 backingGreen;
	CPI_Float32		 backingBlue;
	CPI_Float32		 backingAlpha;
	CPI_Float32		 backingOther;
} CPI_ImageContext;


typedef struct
{
	CPI_ImageContext	 myContext;
	void		 		*data;
} CPI_Image;

/*
// A generic sorting routine definition, which will be used in
// at least the FileID stuff
*/
typedef CPI_Bool			(*CPI_SortFn)( const char	*a,
										   const char	*b );

/* Function needed to create a button */
typedef void				(*CPI_CallBackFn)( const char	*nodename,
											   const char	*parmname,
											   CPI_Float32	 time );

#ifdef __cplusplus
}
#endif

#endif /* _CPI_Types_h_ */

